package ogloszenia.soap;

import jakarta.jws.WebParam;
import jakarta.jws.WebResult;
import jakarta.jws.WebService;
import jakarta.xml.ws.soap.MTOM;
import ogloszenia.Ustawienia;
import ogloszenia.baza.sqlite.DostepDoBazySqlite;
import ogloszenia.baza.sqlite.OgloszenieDAO;
import ogloszenia.exn.BladBazyDanych;
import ogloszenia.exn.NieznanyRekord;
import ogloszenia.model.OgloszenieSamochodowe;

import java.io.IOException;
import java.math.BigDecimal;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;

@WebService(endpointInterface="ogloszenia.soap.Ogloszenia")
public class OgloszeniaImpl {

    public List<OgloszenieSamochodowe> readAll() throws BladBazyDanych {
        try (DostepDoBazySqlite db = DostepDoBazySqlite.newSQLite()) {
            OgloszenieDAO dao = db.newOgloszenieDAO();
            return dao.readAllFull();
        }
    }

    public OgloszenieSamochodowe readOne(int idOgloszenia) throws BladBazyDanych, NieznanyRekord {
        try (DostepDoBazySqlite db = DostepDoBazySqlite.newSQLite()) {
            OgloszenieDAO dao = db.newOgloszenieDAO();
            return dao.findByIdFull(idOgloszenia);
        }
    }

    public List<OgloszenieSamochodowe> readByPrice(
            BigDecimal min,
            BigDecimal max) throws BladBazyDanych {
        try(DostepDoBazySqlite db = DostepDoBazySqlite.newSQLite()) {
            OgloszenieDAO dao = db.newOgloszenieDAO();
            return dao.ogloszeniaWedlugCeny(min, max);
        }
    }

    public void update(
            OgloszenieSamochodowe ogloszenie) throws BladBazyDanych, NieznanyRekord {
        try(DostepDoBazySqlite db = DostepDoBazySqlite.newSQLite()) {
            db.beginTransaction();
            OgloszenieDAO dao = db.newOgloszenieDAO();
            boolean sukces = dao.update(ogloszenie);
            if(!sukces) {
                throw new NieznanyRekord("Brak ogłoszenia nr " + ogloszenie.getIdOgloszenia());
            }
            db.endTransaction(true);
        }
    }

    public int insert(OgloszenieSamochodowe ogloszenie) throws BladBazyDanych {
        // W tej wersji zawsze dodajemy nowy rekord.
        // W razie, gdyby klient przysłał ogłoszenie z ustawionym id, to my i tak to id zignorujemy.
        // W tej wersji w odpowiedzi wysyłamy id nowego rekordu, aby klient dowiedział się jakie ono jest.
        ogloszenie.setIdOgloszenia(null);
        try(DostepDoBazySqlite db = DostepDoBazySqlite.newSQLite()) {
            db.beginTransaction();
            OgloszenieDAO dao = db.newOgloszenieDAO();
            dao.insertNew(ogloszenie);
            db.endTransaction(true);
            return ogloszenie.getIdOgloszenia();
        }
    }

       public byte[] photo(int idOgloszenia) throws NieznanyRekord {
        try {
            Path path = Paths.get(Ustawienia.PHOTOS_PATH, idOgloszenia + ".jpg");
            return Files.readAllBytes(path);
        } catch (IOException e) {
            throw new NieznanyRekord("Brak zdjęcia dla ogłoszenia nr " + idOgloszenia);
        }
    }
}
